/**
 * Copyright (C) 2012 Linutop SAS <http://www.linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ltop-heading.h"

static void ltop_heading_class_init (LtopHeadingClass *klass);
static void ltop_heading_init (LtopHeading *heading);
static void ltop_heading_finalize (GObject *object);

struct _LtopHeadingPrivate
{
    GdkPixbuf *icon;
    gchar     *icon_name;
    gchar     *subtitle;
    gchar     *title;

    GtkWidget *image;
    GtkWidget *label;
};

G_DEFINE_TYPE (LtopHeading, ltop_heading, GTK_TYPE_EVENT_BOX);

static void
ltop_heading_class_init (LtopHeadingClass *klass)
{
    GObjectClass *gobject_class;

    g_type_class_add_private (klass, sizeof (LtopHeadingPrivate));

    gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->finalize = ltop_heading_finalize;
}

static void
ltop_heading_init (LtopHeading *heading)
{
    GtkWidget *box;
    GdkRGBA c = { 255, 255, 255, 100 };

    heading->priv = G_TYPE_INSTANCE_GET_PRIVATE (heading, LTOP_TYPE_HEADING,
                                                 LtopHeadingPrivate);

    heading->priv->image = gtk_image_new ();
    heading->priv->label = gtk_label_new (NULL);
    gtk_label_set_use_markup (GTK_LABEL (heading->priv->label), TRUE);

    box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
    gtk_box_pack_start (GTK_BOX (box), heading->priv->image, FALSE, FALSE, 6);
    gtk_box_pack_start (GTK_BOX (box), heading->priv->label, FALSE, FALSE, 6);

    gtk_widget_override_background_color (GTK_WIDGET (heading),
                                          GTK_STATE_FLAG_NORMAL,
                                          &c);

    gtk_container_add (GTK_CONTAINER (heading), box);
    gtk_widget_show_all (box);
}

static void
ltop_heading_finalize (GObject *object)
{
    LtopHeading *heading = LTOP_HEADING (object);

    if (G_UNLIKELY (heading->priv->icon != NULL))
        g_object_unref (G_OBJECT (heading->priv->icon));
    g_free (heading->priv->icon_name);
    g_free (heading->priv->subtitle);
    g_free (heading->priv->title);

    (*G_OBJECT_CLASS (ltop_heading_parent_class)->finalize) (object);
}

static GdkPixbuf *
ltop_heading_make_pixbuf (LtopHeading *heading)
{
    GtkIconTheme *icon_theme;
    GdkPixbuf    *pixbuf = NULL;
    GdkScreen    *screen;

    if (G_LIKELY (heading->priv->icon_name != NULL))
    {
        screen = gtk_widget_get_screen (GTK_WIDGET (heading));
        icon_theme = gtk_icon_theme_get_for_screen (screen);

        pixbuf = gtk_icon_theme_load_icon (icon_theme, heading->priv->icon_name,
                                           48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
    }

    return pixbuf;
}

/* public */
GtkWidget *
ltop_heading_new ()
{
    return g_object_new (LTOP_TYPE_HEADING, NULL);
}

void
ltop_heading_set_icon (LtopHeading *heading, GdkPixbuf *icon)
{
    if (!icon)
    {
        gtk_widget_hide (heading->priv->image);

        return;
    }

    if (G_LIKELY (heading->priv->icon != NULL))
        g_object_unref (G_OBJECT (heading->priv->icon));

    heading->priv->icon = icon;

    if (G_LIKELY (icon != NULL))
        g_object_ref (G_OBJECT (icon));

    gtk_image_set_from_pixbuf (GTK_IMAGE (heading->priv->image), icon);

    gtk_widget_show (heading->priv->image);
}

void
ltop_heading_set_icon_name (LtopHeading *heading, const gchar *icon_name)
{
    GdkPixbuf *pxb;

    g_free (heading->priv->icon_name);

    if (!icon_name)
    {
        gtk_widget_hide (heading->priv->image);

        return;
    }

    heading->priv->icon_name = g_strdup (icon_name);
    pxb = ltop_heading_make_pixbuf (heading);
    if (pxb)
    {
        ltop_heading_set_icon (heading, pxb);
        g_object_unref (pxb);
    }
    else
    {
        gtk_widget_hide (heading->priv->image);
    }
}

void
ltop_heading_set_label (LtopHeading *heading)
{
    gchar *l;

    if (!heading->priv->title)
        l = g_strdup ("");
    else
    {
        if (!heading->priv->subtitle)
            l = g_strdup_printf ("<b><span size=\"x-large\">%s</span></b>", heading->priv->title);
        else
            l = g_strdup_printf ("<b><span size=\"x-large\">%s</span></b>\n"
                                 "<span style=\"italic\">%s</span>",
                                 heading->priv->title,
                                 heading->priv->subtitle);
    }

    gtk_label_set_markup (GTK_LABEL (heading->priv->label), l);

    g_free (l);
}

void
ltop_heading_set_title (LtopHeading *heading, const gchar *title)
{
    if (!title)
        return;

    g_free (heading->priv->title);
    heading->priv->title = g_strdup (title);
    ltop_heading_set_label (heading);
}

void
ltop_heading_set_subtitle (LtopHeading *heading, const gchar *subtitle)
{
    g_free (heading->priv->subtitle);

    if (subtitle)
    {
        heading->priv->subtitle = g_strdup (subtitle);
        ltop_heading_set_label (heading);
    }
}

