/* mount.c
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

#include <glib/gprintf.h>
#include <gtk/gtk.h>

#include "mount.h"
#include "main.h"

#define SMBCONFFILE "/etc/linutopd/onboot/smbmounts.conf"
#define SMBCMD "sh /usr/lib/linutopd/onboot/01smbmounts"

const gchar *ex[5] = {
    N_("See Windows network properties"),
    N_("Any Windows shared directory"),
    N_("Access directory on Linutop"),
    N_("User owning the share (optional)"),
    N_("Password to acces the share (optional)")
};

/* init a mount structure */
mount *
mount_init ()
{
    mount *m;
    m = g_new0 (mount, 1);

    m->host = g_strdup ("192.168.0.3");
    m->dir = g_strdup ("Documents");
    m->mpoint = g_strdup ("windows");
    m->user = g_strdup ("");
    m->passwd = g_strdup ("");

    return m;
}

void
mount_free (mount *m)
{
    g_free (m->host);
    g_free (m->dir);
    g_free (m->mpoint);
    g_free (m->user);
    g_free (m->passwd);

    g_free (m);
}

void
get_mount_from_form (mount *m)
{
    g_free (m->host);
    m->host = g_strdup (gtk_entry_get_text (GTK_ENTRY (mgui->host_entry)));
    g_free (m->dir);
    m->dir = g_strdup (gtk_entry_get_text (GTK_ENTRY (mgui->dir_entry)));
    g_free (m->mpoint);
    m->mpoint = g_strdup (gtk_entry_get_text (GTK_ENTRY (mgui->mpoint_entry)));
    g_free (m->user);
    m->user = g_strdup (gtk_entry_get_text (GTK_ENTRY (mgui->user_entry)));
    g_free (m->passwd);
    m->passwd = g_strdup (gtk_entry_get_text (GTK_ENTRY (mgui->passwd_entry)));
}

static void
update_mount_in_mgui (GtkTreeIter iter, mount * m)
{
    gtk_list_store_set (GTK_LIST_STORE (mgui->store), &iter,
               S_TYPE_HOST, m->host,
               S_TYPE_DIR, m->dir,
               S_TYPE_MPOINT, m->mpoint,
               S_TYPE_USER, m->user, S_TYPE_PASSWD, m->passwd, -1);
}

static void
append_mount_to_mgui (mount *m)
{
    GtkTreeIter iter;

    gtk_list_store_append (GTK_LIST_STORE (mgui->store), &iter);
    gtk_list_store_set (GTK_LIST_STORE (mgui->store), &iter,
               S_TYPE_HOST, m->host,
               S_TYPE_DIR, m->dir,
               S_TYPE_MPOINT, m->mpoint,
               S_TYPE_USER, m->user, S_TYPE_PASSWD, m->passwd, -1);
}

static void
get_mount_from_mgui (GtkTreeIter iter, mount *m)
{
    gtk_tree_model_get (GTK_TREE_MODEL (mgui->store), &iter,
               S_TYPE_HOST, &m->host,
               S_TYPE_DIR, &m->dir,
               S_TYPE_MPOINT, &m->mpoint,
               S_TYPE_USER, &m->user, S_TYPE_PASSWD, &m->passwd,
               -1);
}

static void
empty_form ()
{
    gtk_entry_set_text (GTK_ENTRY (mgui->host_entry), "");
    gtk_entry_set_text (GTK_ENTRY (mgui->dir_entry), "");
    gtk_entry_set_text (GTK_ENTRY (mgui->mpoint_entry), "");
    gtk_entry_set_text (GTK_ENTRY (mgui->user_entry), "");
    gtk_entry_set_text (GTK_ENTRY (mgui->passwd_entry), "");
}

static void
btns_sensitivity ()
{
    GtkTreeSelection *selection;
    GtkTreeIter iter;
    GtkTreeModel *model;
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (mgui->treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (mgui->treeview));
    if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
        gtk_widget_set_sensitive (mgui->rm_btn, TRUE);
        gtk_widget_set_sensitive (mgui->edit_btn, TRUE);
    } else {
        gtk_widget_set_sensitive (mgui->rm_btn, FALSE);
        gtk_widget_set_sensitive (mgui->edit_btn, FALSE);
    }

    gtk_widget_grab_focus (mgui->host_entry);
}

static void
check_consistency (GtkWidget *w, gpointer data)
{
    mount *m;
    gboolean ok = TRUE;
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreeIter iter;

    m = mount_init ();
    get_mount_from_form (m);

    if (strcmp (m->host, "") == 0)
        ok = FALSE;
    if (strcmp (m->dir, "") == 0)
        ok = FALSE;
    if (strcmp (m->mpoint, "") == 0)
        ok = FALSE;
    g_free (m);

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (mgui->treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (mgui->treeview));

    gtk_widget_set_sensitive (mgui->add_btn, ok);

    ok = ok && gtk_tree_selection_get_selected (selection, &model, &iter);
    gtk_widget_set_sensitive (mgui->edit_btn, ok);
}

static void
set_example ()
{
    gtk_entry_set_text (GTK_ENTRY (mgui->host_entry), "192.168.1.5");
    gtk_entry_set_text (GTK_ENTRY (mgui->dir_entry), "shares");
    gtk_entry_set_text (GTK_ENTRY (mgui->mpoint_entry), "/home/linutop/windows");
}

void
on_item_selected (GtkWidget *w, gpointer data)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreeIter iter;

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (mgui->treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (mgui->treeview));
    if (!selection)
        return;

    mount *m;
    m = mount_init ();

    if (!gtk_tree_selection_get_selected (selection, &model, &iter))
        return;
    get_mount_from_mgui (iter, m);

    gtk_entry_set_text (GTK_ENTRY (mgui->host_entry), m->host);
    gtk_entry_set_text (GTK_ENTRY (mgui->dir_entry), m->dir);
    if (m->mpoint[0] != '\0')
        gtk_entry_set_text (GTK_ENTRY (mgui->mpoint_entry),
                   m->mpoint);
    else
        gtk_entry_set_text (GTK_ENTRY (mgui->mpoint_entry),
                   "/home/linutop/windows");
    gtk_entry_set_text (GTK_ENTRY (mgui->user_entry), m->user);
    gtk_entry_set_text (GTK_ENTRY (mgui->passwd_entry), m->passwd);

    mount_free (m);

    btns_sensitivity ();
}

void
mount_save ()
{
    FILE *fp;
    GtkTreeIter iter;
    gboolean get;
    mount *m;

    m = mount_init ();
    fp = fopen (SMBCONFFILE, "w");
    if (!fp)
        return;

    get = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mgui->store), &iter);
    while (get) {
        get_mount_from_mgui (iter, m);
        g_fprintf (fp, "%s %s %s %s %s\n", m->host,
              m->dir, m->mpoint, m->user, m->passwd);
        get =
            gtk_tree_model_iter_next (GTK_TREE_MODEL (mgui->store),
                         &iter);
    }
    fclose (fp);
    chmod (SMBCONFFILE, 0600);
    if (m)
        mount_free (m);

    g_spawn_command_line_async (SMBCMD, NULL);
}

/*
 * main dialog callbacks
 */
static void
add_btn_clicked (GtkWidget *w, gpointer data)
{
    mount *m;

    m = mount_init ();
    get_mount_from_form (m);
    append_mount_to_mgui (m);
    empty_form ();
    mount_free (m);

    btns_sensitivity ();
}

static void
edit_btn_clicked (GtkWidget *w, gpointer data)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (mgui->treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (mgui->treeview));
    if (selection) {
        GtkTreeIter iter;
        mount *m;
        m = mount_init ();

        gtk_tree_selection_get_selected (selection, &model, &iter);
        get_mount_from_mgui (iter, m);
        get_mount_from_form (m);
        update_mount_in_mgui (iter, m);
        mount_free (m);
    }

    btns_sensitivity ();
}

static void
rm_btn_clicked (GtkWidget *w, gpointer data)
{
    GtkTreeSelection *selection;
    GtkTreeIter iter;
    GtkTreeModel *model;
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (mgui->treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (mgui->treeview));
    if (selection) {
        gtk_tree_selection_get_selected (selection, &model, &iter);
        gtk_list_store_remove (mgui->store, &iter);
        mgui->items_nb--;
    }

    empty_form ();
    btns_sensitivity ();
}

/* read the config and populate the main treeview */
void
populate_treestore ()
{
    FILE *fp;
    gchar line[400];
    mount *m;
    gchar **tmp = NULL;
    guint tokens;

    m = mount_init ();

    fp = fopen (SMBCONFFILE, "r");
    if (!fp)
        return;
    while (fgets (line, 399, fp) != NULL) {
        if (line[0] && (line[0] != '\n')
            && (line[strlen (line) - 1] == '\n'))
            line[strlen (line) - 1] = '\0';
        else
            continue;
        tmp = g_strsplit (line, " ", 5);
        tokens = g_strv_length (tmp);
        if ( (tokens > 5) && (tokens < 3)) {
            continue;
        }
        m->host = strdup (tmp[0]);
        m->dir = strdup (tmp[1]);
        m->mpoint = strdup (tmp[2]);
        if (tokens >= 4)
            m->user = strdup (tmp[3]);
        else
            m->user = strdup ("");
        if (tokens == 5)
            m->passwd = strdup (tmp[4]);
        else
            m->passwd = strdup ("");
        append_mount_to_mgui (m);
        mgui->items_nb++;

        g_strfreev (tmp);
        tmp = NULL;
    }
    mount_free (m);
    fclose (fp);
}

static void
free_all (GtkWidget *w, gpointer data)
{
    g_free (data);
}

static void
on_dialog_delete (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    return;
}

static void
on_dialog_response (GtkWidget *dialog, gint response_id, gpointer data)
{
    gtk_widget_hide (dialog);
}

/* create the main gui */
GtkWidget *
create_mount_gui ()
{
    int i;
    int row;
    GtkWidget *hbox, *vbox, *sw;
    GtkWidget *table, *label;
    GtkWidget *mount_dialog;
    GtkWidget *content_area;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    gchar *markup = NULL;

    mgui = g_new0 (main_gui, 1);

    mount_dialog = xfce_titled_dialog_new_with_buttons (
                            _("Windows shares access"),
                            GTK_WINDOW (dialog->dialog),
                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                            NULL);
    content_area = gtk_dialog_get_content_area (GTK_DIALOG (mount_dialog));
    gtk_window_set_icon_name (GTK_WINDOW (mount_dialog), "network-receive");
    xfce_titled_dialog_set_subtitle (XFCE_TITLED_DIALOG (mount_dialog),
                                     _("Remotely access Windows shared folders"));
    g_signal_connect (G_OBJECT (mount_dialog), "response",
                      G_CALLBACK (on_dialog_response), NULL);
    g_signal_connect (G_OBJECT (mount_dialog), "delete-event",
                      G_CALLBACK (on_dialog_delete), NULL);

    sw = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (content_area), sw, TRUE, TRUE, SPACE);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                       GTK_POLICY_AUTOMATIC,
                       GTK_POLICY_AUTOMATIC);

    mgui->store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING,
                     G_TYPE_STRING, G_TYPE_STRING,
                     G_TYPE_STRING);

    mgui->treeview =
        gtk_tree_view_new_with_model (GTK_TREE_MODEL (mgui->store));
    for (i = 0; i < 4; i++) {
        renderer = gtk_cell_renderer_text_new ();
        column = gtk_tree_view_column_new_with_attributes (_(columns[i]),
                                  renderer,
                                  "text", i,
                                  NULL);
        gtk_tree_view_column_set_resizable (column, TRUE);
        gtk_tree_view_append_column (GTK_TREE_VIEW (mgui->treeview),
                        column);
    }
    gtk_container_add (GTK_CONTAINER (sw), mgui->treeview);


    /*
     * Add/Modify section
     */
    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, SPACE);
    gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, SPACE);

    table = gtk_table_new (6, 3, FALSE);
    gtk_box_pack_start (GTK_BOX (hbox), table, FALSE, FALSE, 2);

    /* header */
    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Property</span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
            GTK_FILL, GTK_FILL, 2, 2);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Value</span>"));
    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
            GTK_FILL, GTK_FILL, 2, 2);

    /* content */
    label = gtk_label_new (_("Host:"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
             GTK_FILL, GTK_FILL, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

    mgui->host_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), mgui->host_entry, 1, 2, 1, 2,
             GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);

    label = gtk_label_new (_("Directory:"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
             GTK_FILL, GTK_FILL, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

    mgui->dir_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), mgui->dir_entry, 1, 2, 2, 3,
             GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);

    label = gtk_label_new (_("Mount point:"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
             GTK_FILL, GTK_FILL, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

    mgui->mpoint_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), mgui->mpoint_entry,
             1, 2, 3, 4,
             GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);

    label = gtk_label_new (_("User:"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
             GTK_FILL, GTK_FILL, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

    mgui->user_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), mgui->user_entry,
             1, 2, 4, 5,
             GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);

    label = gtk_label_new (_("Password:"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6,
             GTK_FILL, GTK_FILL, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

    mgui->passwd_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), mgui->passwd_entry,
             1, 2, 5, 6,
             GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);
    gtk_entry_set_visibility (GTK_ENTRY (mgui->passwd_entry), FALSE);

    /* examples */
    for (row = 0; row < 5; row++)
    {
        label = gtk_label_new ("");
        markup = g_markup_printf_escaped (
            "<span size=\"small\" style=\"italic\">%s</span>", _(ex[row]));
        gtk_label_set_markup (GTK_LABEL (label), markup);
        g_free (markup);
        markup = NULL;
        gtk_table_attach (GTK_TABLE (table),
            label, 2, 3, row+1, row+2,
            GTK_FILL, GTK_FILL, 6, 6);
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    }

    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, SPACE);
    gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, SPACE);

    mgui->add_btn = gtk_button_new_from_stock (GTK_STOCK_ADD);
    gtk_widget_set_sensitive (mgui->add_btn, FALSE);
    gtk_box_pack_start (GTK_BOX (vbox), mgui->add_btn, FALSE, FALSE, SPACE);
    mgui->edit_btn = gtk_button_new_from_stock (GTK_STOCK_EDIT);
    gtk_button_set_label (GTK_BUTTON (mgui->edit_btn), _("Validate"));
    gtk_button_set_image (GTK_BUTTON (mgui->edit_btn),
                          gtk_image_new_from_icon_name ("linutop-green-ok",
                                                        GTK_ICON_SIZE_BUTTON));
    gtk_widget_set_sensitive (mgui->edit_btn, FALSE);
    gtk_box_pack_start (GTK_BOX (vbox), mgui->edit_btn, FALSE, FALSE, SPACE);
    mgui->rm_btn = gtk_button_new_from_stock (GTK_STOCK_DELETE);
    gtk_widget_set_sensitive (mgui->rm_btn, FALSE);
    gtk_box_pack_start (GTK_BOX (vbox), mgui->rm_btn, FALSE, FALSE, SPACE);

    g_signal_connect (G_OBJECT (mgui->add_btn), "clicked",
             G_CALLBACK (add_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (mgui->edit_btn), "clicked",
             G_CALLBACK (edit_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (mgui->rm_btn), "clicked",
             G_CALLBACK (rm_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (mgui->treeview), "cursor-changed",
             G_CALLBACK (on_item_selected), NULL);

    g_signal_connect (G_OBJECT (mgui->host_entry), "changed",
             G_CALLBACK (check_consistency), (gpointer) NULL);
    g_signal_connect (G_OBJECT (mgui->dir_entry), "changed",
             G_CALLBACK (check_consistency), (gpointer) NULL);
    g_signal_connect (G_OBJECT (mgui->mpoint_entry), "changed",
             G_CALLBACK (check_consistency), (gpointer) NULL);

    g_signal_connect (G_OBJECT (content_area), "destroy",
                      G_CALLBACK (free_all), mgui);

    populate_treestore ();

    if (!mgui->items_nb)
        set_example ();

    gtk_widget_grab_focus (mgui->host_entry);
    gtk_widget_show_all (content_area);

    return mount_dialog;
}
